import { AD_GET_BASIC_ROUTE_CONTROLLER } from '@modules/ad/ad.di-tokens';
import { CarpoolRoute } from '@modules/ad/core/application/types/carpool-route.type';
import { Role } from '@modules/ad/core/domain/ad.types';
import { CarpoolRouteProvider } from '@modules/ad/infrastructure/carpool-route-provider';
import { GetBasicRouteControllerPort } from '@modules/geography/core/application/ports/get-basic-route-controller.port';
import { Test, TestingModule } from '@nestjs/testing';

const mockGetBasicRouteController: GetBasicRouteControllerPort = {
  get: jest.fn().mockImplementation(() => ({
    driverDistance: 23000,
    driverDuration: 900,
    passengerDistance: 23000,
    passengerDuration: 900,
    fwdAzimuth: 283,
    backAzimuth: 93,
    distanceAzimuth: 19840,
    points: [
      {
        lon: 6.1765103,
        lat: 48.689446,
      },
      {
        lon: 2.3523,
        lat: 48.8567,
      },
    ],
  })),
};

describe('Carpool route provider', () => {
  let carpoolRouteProvider: CarpoolRouteProvider;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        CarpoolRouteProvider,
        {
          provide: AD_GET_BASIC_ROUTE_CONTROLLER,
          useValue: mockGetBasicRouteController,
        },
      ],
    }).compile();

    carpoolRouteProvider =
      module.get<CarpoolRouteProvider>(CarpoolRouteProvider);
  });

  it('should be defined', () => {
    expect(carpoolRouteProvider).toBeDefined();
  });

  it('should provide a carpool route', async () => {
    const carpoolRoute: CarpoolRoute = await carpoolRouteProvider.getBasic(
      [Role.DRIVER],
      [
        {
          position: 0,
          lat: 48.689445,
          lon: 6.1765102,
        },
        {
          position: 1,
          lat: 48.8566,
          lon: 2.3522,
        },
      ],
    );
    expect(carpoolRoute.driverDistance).toBe(23000);
  });
});
